@@ -85,7 +85,7 @@ gssanova <- function(formula,family,type="cubic",data=list(),
         desc <- rbind(desc,as.numeric(c(term[[label]][c("nphi","nrk")])))
     desc <- rbind(desc,apply(desc,2,sum))
     rownames(desc) <- c(term$labels,"total")
-    colnames(desc) <- c("Fixed","Random")
+    colnames(desc) <- c("Unpenalized","Penalized")
     ## Return the results
     obj <- c(list(call=match.call(),family=family,mf=mf,terms=term,desc=desc),z)
     class(obj) <- c("gssanova","ssanova")
